@@ -1,9 +1,9 @@
 /* kwset.c - search for any of a set of keywords.
-   Copyright 1989, 1998, 2000 Free Software Foundation, Inc.
+   Copyright 1989, 1998, 2000, 2005 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
+   the Free Software Foundation; either version 3, or (at your option)
    any later version.
 
    This program is distributed in the hope that it will be useful,
@@ -13,8 +13,8 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
-   02111-1307, USA.  */
+   Foundation, Inc., 51 Franklin Street - Fifth Floor, Boston, MA
+   02110-1301, USA.  */
 
 /* Written August 1989 by Mike Haertel.
    The author may be reached (Email) at the address mike@ai.mit.edu,
@@ -46,6 +46,8 @@ extern char *xmalloc();
 #define obstack_chunk_alloc malloc
 #define obstack_chunk_free free
 
+#define U(c) ((unsigned char) (c))
+
 /* Balanced tree of edges and labels leaving a given trie node. */
 struct tree
 {
@@ -81,28 +83,19 @@ struct kwset
   struct trie *next[NCHAR];	/* Table of children of the root. */
   char *target;			/* Target string if there's only one. */
   int mind2;			/* Used in Boyer-Moore search for one string. */
-  char *trans;			/* Character translation table. */
+  char const *trans;		/* Character translation table. */
 };
 
-/* prototypes */
-static void enqueue PARAMS((struct tree *, struct trie **));
-static void treefails PARAMS((register struct tree *, struct trie *, struct trie *));
-static void treedelta PARAMS((register struct tree *,register unsigned int, unsigned char *));
-static int  hasevery PARAMS((register struct tree *, register struct tree *));
-static void treenext PARAMS((struct tree *, struct trie **));
-static char * bmexec PARAMS((kwset_t, char *, size_t));
-static char * cwexec PARAMS((kwset_t, char *, size_t, struct kwsmatch *));
-
 /* Allocate and initialize a keyword set object, returning an opaque
    pointer to it.  Return NULL if memory is not available. */
 kwset_t
-kwsalloc (char *trans)
+kwsalloc (char const *trans)
 {
   struct kwset *kwset;
 
   kwset = (struct kwset *) malloc(sizeof (struct kwset));
   if (!kwset)
-    return 0;
+    return NULL;
 
   obstack_init(&kwset->obstack);
   kwset->words = 0;
@@ -111,35 +104,39 @@ kwsalloc (char *trans)
   if (!kwset->trie)
     {
       kwsfree((kwset_t) kwset);
-      return 0;
+      return NULL;
     }
   kwset->trie->accepting = 0;
-  kwset->trie->links = 0;
-  kwset->trie->parent = 0;
-  kwset->trie->next = 0;
-  kwset->trie->fail = 0;
+  kwset->trie->links = NULL;
+  kwset->trie->parent = NULL;
+  kwset->trie->next = NULL;
+  kwset->trie->fail = NULL;
   kwset->trie->depth = 0;
   kwset->trie->shift = 0;
   kwset->mind = INT_MAX;
   kwset->maxd = -1;
-  kwset->target = 0;
+  kwset->target = NULL;
   kwset->trans = trans;
 
   return (kwset_t) kwset;
 }
 
+/* This upper bound is valid for CHAR_BIT >= 4 and
+   exact for CHAR_BIT in { 4..11, 13, 15, 17, 19 }. */
+#define DEPTH_SIZE (CHAR_BIT + CHAR_BIT/2)
+
 /* Add the given string to the contents of the keyword set.  Return NULL
    for success, an error message otherwise. */
-char *
-kwsincr (kwset_t kws, char *text, size_t len)
+const char *
+kwsincr (kwset_t kws, char const *text, size_t len)
 {
   struct kwset *kwset;
   register struct trie *trie;
   register unsigned char label;
   register struct tree *link;
   register int depth;
-  struct tree *links[12];
-  enum { L, R } dirs[12];
+  struct tree *links[DEPTH_SIZE];
+  enum { L, R } dirs[DEPTH_SIZE];
   struct tree *t, *r, *l, *rl, *lr;
 
   kwset = (struct kwset *) kws;
@@ -150,7 +147,7 @@ kwsincr (kwset_t kws, char *text, size_t len)
      installing new nodes when necessary. */
   while (len--)
     {
-      label = kwset->trans ? kwset->trans[(unsigned char) *--text] : *--text;
+      label = kwset->trans ? kwset->trans[U(*--text)] : *--text;
 
       /* Descend the tree of outgoing links for this trie node,
 	 looking for the current character and keeping track
@@ -178,17 +175,20 @@ kwsincr (kwset_t kws, char *text, size_t len)
 					       sizeof (struct tree));
 	  if (!link)
 	    return _("memory exhausted");
-	  link->llink = 0;
-	  link->rlink = 0;
+	  link->llink = NULL;
+	  link->rlink = NULL;
 	  link->trie = (struct trie *) obstack_alloc(&kwset->obstack,
 						     sizeof (struct trie));
 	  if (!link->trie)
-	    return _("memory exhausted");
+	    {
+	      obstack_free(&kwset->obstack, link);
+	      return _("memory exhausted");
+	    }
 	  link->trie->accepting = 0;
-	  link->trie->links = 0;
+	  link->trie->links = NULL;
 	  link->trie->parent = trie;
-	  link->trie->next = 0;
-	  link->trie->fail = 0;
+	  link->trie->next = NULL;
+	  link->trie->fail = NULL;
 	  link->trie->depth = trie->depth + 1;
 	  link->trie->shift = 0;
 	  link->label = label;
@@ -282,7 +282,7 @@ kwsincr (kwset_t kws, char *text, size_t len)
   if (trie->depth > kwset->maxd)
     kwset->maxd = trie->depth;
 
-  return 0;
+  return NULL;
 }
 
 /* Enqueue the trie nodes referenced from the given tree in the
@@ -301,7 +301,8 @@ enqueue (struct tree *tree, struct trie **last)
    from the given tree, given the failure function for their parent as
    well as a last resort failure node. */
 static void
-treefails (register struct tree *tree, struct trie *fail, struct trie *recourse)
+treefails (register struct tree const *tree, struct trie const *fail,
+	   struct trie *recourse)
 {
   register struct tree *link;
 
@@ -335,7 +336,7 @@ treefails (register struct tree *tree, struct trie *fail, struct trie *recourse)
 /* Set delta entries for the links of the given tree such that
    the preexisting delta value is larger than the current depth. */
 static void
-treedelta (register struct tree *tree,
+treedelta (register struct tree const *tree,
 	   register unsigned int depth,
 	   unsigned char delta[])
 {
@@ -349,7 +350,7 @@ treedelta (register struct tree *tree,
 
 /* Return true if A has every label in B. */
 static int
-hasevery (register struct tree *a, register struct tree *b)
+hasevery (register struct tree const *a, register struct tree const *b)
 {
   if (!b)
     return 1;
@@ -368,7 +369,7 @@ hasevery (register struct tree *a, register struct tree *b)
 /* Compute a vector, indexed by character code, of the trie nodes
    referenced from the given tree. */
 static void
-treenext (struct tree *tree, struct trie *next[])
+treenext (struct tree const *tree, struct trie *next[])
 {
   if (!tree)
     return;
@@ -379,34 +380,32 @@ treenext (struct tree *tree, struct trie *next[])
 
 /* Compute the shift for each trie node, as well as the delta
    table and next cache for the given keyword set. */
-char *
+const char *
 kwsprep (kwset_t kws)
 {
   register struct kwset *kwset;
   register int i;
-  register struct trie *curr, *fail;
-  register char *trans;
+  register struct trie *curr;
+  register char const *trans;
   unsigned char delta[NCHAR];
-  struct trie *last, *next[NCHAR];
 
   kwset = (struct kwset *) kws;
 
   /* Initial values for the delta table; will be changed later.  The
      delta entry for a given character is the smallest depth of any
      node at which an outgoing edge is labeled by that character. */
-  if (kwset->mind < 256)
-    for (i = 0; i < NCHAR; ++i)
-      delta[i] = kwset->mind;
-  else
-    for (i = 0; i < NCHAR; ++i)
-      delta[i] = 255;
+  memset(delta, kwset->mind < UCHAR_MAX ? kwset->mind : UCHAR_MAX, NCHAR);
 
   /* Check if we can use the simple boyer-moore algorithm, instead
      of the hairy commentz-walter algorithm. */
-  if (kwset->words == 1 && kwset->trans == 0)
+  if (kwset->words == 1 && kwset->trans == NULL)
     {
+      char c;
+
       /* Looking for just one string.  Extract it from the trie. */
       kwset->target = obstack_alloc(&kwset->obstack, kwset->mind);
+      if (!kwset->target)
+	return _("memory exhausted");
       for (i = kwset->mind - 1, curr = kwset->trie; i >= 0; --i)
 	{
 	  kwset->target[i] = curr->links->label;
@@ -414,16 +413,20 @@ kwsprep (kwset_t kws)
 	}
       /* Build the Boyer Moore delta.  Boy that's easy compared to CW. */
       for (i = 0; i < kwset->mind; ++i)
-	delta[(unsigned char) kwset->target[i]] = kwset->mind - (i + 1);
-      kwset->mind2 = kwset->mind;
+	delta[U(kwset->target[i])] = kwset->mind - (i + 1);
       /* Find the minimal delta2 shift that we might make after
 	 a backwards match has failed. */
-      for (i = 0; i < kwset->mind - 1; ++i)
-	if (kwset->target[i] == kwset->target[kwset->mind - 1])
-	  kwset->mind2 = kwset->mind - (i + 1);
+      c = kwset->target[kwset->mind - 1];
+      for (i = kwset->mind - 2; i >= 0; --i)
+	if (kwset->target[i] == c)
+	  break;
+      kwset->mind2 = kwset->mind - (i + 1);
     }
   else
     {
+      register struct trie *fail;
+      struct trie *last, *next[NCHAR];
+
       /* Traverse the nodes of the trie in level order, simultaneously
 	 computing the delta table, failure function, and shift function. */
       for (curr = last = kwset->trie; curr; curr = curr->next)
@@ -472,48 +475,47 @@ kwsprep (kwset_t kws)
       /* Create a vector, indexed by character code, of the outgoing links
 	 from the root node. */
       for (i = 0; i < NCHAR; ++i)
-	next[i] = 0;
+	next[i] = NULL;
       treenext(kwset->trie->links, next);
 
-      if ((trans = kwset->trans) != 0)
+      if ((trans = kwset->trans) != NULL)
 	for (i = 0; i < NCHAR; ++i)
-	  kwset->next[i] = next[(unsigned char) trans[i]];
+	  kwset->next[i] = next[U(trans[i])];
       else
-	for (i = 0; i < NCHAR; ++i)
-	  kwset->next[i] = next[i];
+	memcpy(kwset->next, next, NCHAR * sizeof(struct trie *));
     }
 
   /* Fix things up for any translation table. */
-  if ((trans = kwset->trans) != 0)
+  if ((trans = kwset->trans) != NULL)
     for (i = 0; i < NCHAR; ++i)
-      kwset->delta[i] = delta[(unsigned char) trans[i]];
+      kwset->delta[i] = delta[U(trans[i])];
   else
-    for (i = 0; i < NCHAR; ++i)
-      kwset->delta[i] = delta[i];
+    memcpy(kwset->delta, delta, NCHAR);
 
-  return 0;
+  return NULL;
 }
 
-#define U(C) ((unsigned char) (C))
-
 /* Fast boyer-moore search. */
-static char *
-bmexec (kwset_t kws, char *text, size_t size)
+static size_t
+bmexec (kwset_t kws, char const *text, size_t size)
 {
-  struct kwset *kwset;
-  register unsigned char *d1;
-  register char *ep, *sp, *tp;
+  struct kwset const *kwset;
+  register unsigned char const *d1;
+  register char const *ep, *sp, *tp;
   register int d, gc, i, len, md2;
 
-  kwset = (struct kwset *) kws;
+  kwset = (struct kwset const *) kws;
   len = kwset->mind;
 
   if (len == 0)
-    return text;
-  if (len > size)
     return 0;
+  if (len > size)
+    return -1;
   if (len == 1)
-    return memchr(text, kwset->target[0], size);
+    {
+      tp = memchr (text, kwset->target[0], size);
+      return tp ? tp - text : -1;
+    }
 
   d1 = kwset->delta;
   sp = kwset->target + len;
@@ -552,7 +554,7 @@ bmexec (kwset_t kws, char *text, size_t size)
 	    for (i = 3; i <= len && U(tp[-i]) == U(sp[-i]); ++i)
 	      ;
 	    if (i > len)
-	      return tp - len;
+	      return tp - len - text;
 	  }
 	tp += md2;
       }
@@ -571,26 +573,29 @@ bmexec (kwset_t kws, char *text, size_t size)
 	  for (i = 3; i <= len && U(tp[-i]) == U(sp[-i]); ++i)
 	    ;
 	  if (i > len)
-	    return tp - len;
+	    return tp - len - text;
 	}
       d = md2;
     }
 
-  return 0;
+  return -1;
 }
 
 /* Hairy multiple string search. */
-static char *
-cwexec (kwset_t kws, char *text, size_t len, struct kwsmatch *kwsmatch)
+static size_t
+cwexec (kwset_t kws, char const *text, size_t len, struct kwsmatch *kwsmatch)
 {
-  struct kwset *kwset;
-  struct trie **next, *trie, *accept;
-  char *beg, *lim, *mch, *lmch;
-  register unsigned char c, *delta;
+  struct kwset const *kwset;
+  struct trie * const *next;
+  struct trie const *trie;
+  struct trie const *accept;
+  char const *beg, *lim, *mch, *lmch;
+  register unsigned char c;
+  register unsigned char const *delta;
   register int d;
-  register char *end, *qlim;
-  register struct tree *tree;
-  register char *trans;
+  register char const *end, *qlim;
+  register struct tree const *tree;
+  register char const *trans;
 
 #ifdef lint
   accept = NULL;
@@ -599,14 +604,14 @@ cwexec (kwset_t kws, char *text, size_t len, struct kwsmatch *kwsmatch)
   /* Initialize register copies and look for easy ways out. */
   kwset = (struct kwset *) kws;
   if (len < kwset->mind)
-    return 0;
+    return -1;
   next = kwset->next;
   delta = kwset->delta;
   trans = kwset->trans;
   lim = text + len;
   end = text;
   if ((d = kwset->mind) != 0)
-    mch = 0;
+    mch = NULL;
   else
     {
       mch = text, accept = kwset->trie;
@@ -616,7 +621,7 @@ cwexec (kwset_t kws, char *text, size_t len, struct kwsmatch *kwsmatch)
   if (len >= 4 * kwset->mind)
     qlim = lim - 4 * kwset->mind;
   else
-    qlim = 0;
+    qlim = NULL;
 
   while (lim - end >= d)
     {
@@ -626,8 +631,8 @@ cwexec (kwset_t kws, char *text, size_t len, struct kwsmatch *kwsmatch)
 	  while ((d = delta[c = *end]) && end < qlim)
 	    {
 	      end += d;
-	      end += delta[(unsigned char) *end];
-	      end += delta[(unsigned char) *end];
+	      end += delta[U(*end)];
+	      end += delta[U(*end)];
 	    }
 	  ++end;
 	}
@@ -645,7 +650,7 @@ cwexec (kwset_t kws, char *text, size_t len, struct kwsmatch *kwsmatch)
       d = trie->shift;
       while (beg > text)
 	{
-	  c = trans ? trans[(unsigned char) *--beg] : *--beg;
+	  c = trans ? trans[U(*--beg)] : *--beg;
 	  tree = trie->links;
 	  while (tree && c != tree->label)
 	    if (c < tree->label)
@@ -668,7 +673,7 @@ cwexec (kwset_t kws, char *text, size_t len, struct kwsmatch *kwsmatch)
       if (mch)
 	goto match;
     }
-  return 0;
+  return -1;
 
  match:
   /* Given a known match, find the longest possible match anchored
@@ -696,7 +701,7 @@ cwexec (kwset_t kws, char *text, size_t len, struct kwsmatch *kwsmatch)
       d = trie->shift;
       while (beg > text)
 	{
-	  c = trans ? trans[(unsigned char) *--beg] : *--beg;
+	  c = trans ? trans[U(*--beg)] : *--beg;
 	  tree = trie->links;
 	  while (tree && c != tree->label)
 	    if (c < tree->label)
@@ -728,10 +733,10 @@ cwexec (kwset_t kws, char *text, size_t len, struct kwsmatch *kwsmatch)
   if (kwsmatch)
     {
       kwsmatch->index = accept->accepting / 2;
-      kwsmatch->beg[0] = mch;
+      kwsmatch->offset[0] = mch - text;
       kwsmatch->size[0] = accept->depth;
     }
-  return mch;
+  return mch - text;
 }
 
 /* Search through the given text for a match of any member of the
@@ -741,20 +746,18 @@ cwexec (kwset_t kws, char *text, size_t len, struct kwsmatch *kwsmatch)
    matching substring.  Similarly, if FOUNDIDX is non-NULL, store
    in the referenced location the index number of the particular
    keyword matched. */
-char *
-kwsexec (kwset_t kws, char *text, size_t size, struct kwsmatch *kwsmatch)
+size_t
+kwsexec (kwset_t kws, char const *text, size_t size,
+	 struct kwsmatch *kwsmatch)
 {
-  struct kwset *kwset;
-  char *ret;
-
-  kwset = (struct kwset *) kws;
-  if (kwset->words == 1 && kwset->trans == 0)
+  struct kwset const *kwset = (struct kwset *) kws;
+  if (kwset->words == 1 && kwset->trans == NULL)
     {
-      ret = bmexec(kws, text, size);
-      if (kwsmatch != 0 && ret != 0)
+      size_t ret = bmexec (kws, text, size);
+      if (kwsmatch != NULL && ret != (size_t) -1)
 	{
 	  kwsmatch->index = 0;
-	  kwsmatch->beg[0] = ret;
+	  kwsmatch->offset[0] = ret;
 	  kwsmatch->size[0] = kwset->mind;
 	}
       return ret;
@@ -770,6 +773,6 @@ kwsfree (kwset_t kws)
   struct kwset *kwset;
 
   kwset = (struct kwset *) kws;
-  obstack_free(&kwset->obstack, 0);
+  obstack_free(&kwset->obstack, NULL);
   free(kws);
 }
