@@ -1,9 +1,9 @@
 /* kwset.h - header declaring the keyword set library.
-   Copyright (C) 1989, 1998 Free Software Foundation, Inc.
+   Copyright (C) 1989, 1998, 2005 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
+   the Free Software Foundation; either version 3, or (at your option)
    any later version.
 
    This program is distributed in the hope that it will be useful,
@@ -13,8 +13,8 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
-   02111-1307, USA.  */
+   Foundation, Inc., 51 Franklin Street - Fifth Floor, Boston, MA
+   02110-1301, USA.  */
 
 /* Written August 1989 by Mike Haertel.
    The author may be reached (Email) at the address mike@ai.mit.edu,
@@ -38,11 +38,11 @@ extern kwset_t kwsalloc PARAMS((char const *));
 /* Incrementally extend the keyword set to include the given string.
    Return NULL for success, or an error message.  Remember an index
    number for each keyword included in the set. */
-extern char *kwsincr PARAMS((kwset_t, char const *, size_t));
+extern const char *kwsincr PARAMS((kwset_t, char const *, size_t));
 
 /* When the keyword set has been completely built, prepare it for
    use.  Return NULL for success, or an error message. */
-extern char *kwsprep PARAMS((kwset_t));
+extern const char *kwsprep PARAMS((kwset_t));
 
 /* Search through the given buffer for a member of the keyword set.
    Return a pointer to the leftmost longest match found, or NULL if
